@@ -11,7 +11,13 @@ if this_dir not in sys.path:
     sys.path.insert(0, this_dir) # needed for Py3
 
 from common_imports import etree, BytesIO, _bytes, HelperTestCase, fileInTestDir
-from common_imports import doctest, make_doctest
+from common_imports import doctest, make_doctest, skipif
+
+try:
+    import rnc2rng
+except ImportError:
+    rnc2rng = None
+
 
 class ETreeRelaxNGTestCase(HelperTestCase):
     def test_relaxng(self):
@@ -27,8 +33,14 @@ class ETreeRelaxNGTestCase(HelperTestCase):
 </element>
 ''')
         schema = etree.RelaxNG(schema)
-        self.assert_(schema.validate(tree_valid))
-        self.assert_(not schema.validate(tree_invalid))
+        self.assertTrue(schema.validate(tree_valid))
+        self.assertFalse(schema.error_log.filter_from_errors())
+
+        self.assertFalse(schema.validate(tree_invalid))
+        self.assertTrue(schema.error_log.filter_from_errors())
+
+        self.assertTrue(schema.validate(tree_valid))             # repeat valid
+        self.assertFalse(schema.error_log.filter_from_errors())  # repeat valid
 
     def test_relaxng_stringio(self):
         tree_valid = self.parse('<a><b></b></a>')
@@ -43,8 +55,8 @@ class ETreeRelaxNGTestCase(HelperTestCase):
 </element>
 ''')
         schema = etree.RelaxNG(file=schema_file)
-        self.assert_(schema.validate(tree_valid))
-        self.assert_(not schema.validate(tree_invalid))
+        self.assertTrue(schema.validate(tree_valid))
+        self.assertFalse(schema.validate(tree_invalid))
 
     def test_relaxng_elementtree_error(self):
         self.assertRaises(ValueError, etree.RelaxNG, etree.ElementTree())
@@ -61,12 +73,12 @@ class ETreeRelaxNGTestCase(HelperTestCase):
 </element>
 ''')
         schema = etree.RelaxNG(schema)
-        self.assert_(not schema.validate(tree_invalid))
+        self.assertFalse(schema.validate(tree_invalid))
         errors = schema.error_log
-        self.assert_([ log for log in errors
-                       if log.level_name == "ERROR" ])
-        self.assert_([ log for log in errors
-                       if "not expect" in log.message ])
+        self.assertTrue([log for log in errors
+                         if log.level_name == "ERROR"])
+        self.assertTrue([log for log in errors
+                         if "not expect" in log.message])
 
     def test_relaxng_invalid_schema(self):
         schema = self.parse('''\
@@ -126,8 +138,8 @@ class ETreeRelaxNGTestCase(HelperTestCase):
   </zeroOrMore>
 </element>
 ''')
-        self.assert_(tree_valid.relaxng(schema))
-        self.assert_(not tree_invalid.relaxng(schema))
+        self.assertTrue(tree_valid.relaxng(schema))
+        self.assertFalse(tree_invalid.relaxng(schema))
 
     def test_multiple_elementrees(self):
         tree = self.parse('<a><b>B</b><c>C</c></a>')
@@ -141,8 +153,11 @@ class ETreeRelaxNGTestCase(HelperTestCase):
   </element>
 </element>
 ''') )
-        self.assert_(schema.validate(tree))
-        self.assert_(schema.validate(tree))
+        self.assertTrue(schema.validate(tree))
+        self.assertFalse(schema.error_log.filter_from_errors())
+
+        self.assertTrue(schema.validate(tree))                   # repeat valid
+        self.assertFalse(schema.error_log.filter_from_errors())  # repeat valid
 
         schema = etree.RelaxNG( self.parse('''\
 <element name="b" xmlns="http://relaxng.org/ns/structure/1.0">
@@ -151,11 +166,37 @@ class ETreeRelaxNGTestCase(HelperTestCase):
 ''') )
         c_tree = etree.ElementTree(tree.getroot()[1])
         self.assertEqual(self._rootstring(c_tree), _bytes('<c>C</c>'))
-        self.assert_(not schema.validate(c_tree))
+        self.assertFalse(schema.validate(c_tree))
+        self.assertTrue(schema.error_log.filter_from_errors())
 
         b_tree = etree.ElementTree(tree.getroot()[0])
         self.assertEqual(self._rootstring(b_tree), _bytes('<b>B</b>'))
-        self.assert_(schema.validate(b_tree))
+        self.assertTrue(schema.validate(b_tree))
+        self.assertFalse(schema.error_log.filter_from_errors())
+
+class RelaxNGCompactTestCase(HelperTestCase):
+
+    pytestmark = skipif('rnc2rng is None')
+
+    def test_relaxng_compact(self):
+        tree_valid = self.parse('<a><b>B</b><c>C</c></a>')
+        tree_invalid = self.parse('<a><b></b></a>')
+        schema = etree.RelaxNG(file=fileInTestDir('test.rnc'))
+        self.assertTrue(schema.validate(tree_valid))
+        self.assertFalse(schema.validate(tree_invalid))
+
+    def test_relaxng_compact_file_obj(self):
+        f = open(fileInTestDir('test.rnc'), 'rb')
+        try:
+            schema = etree.RelaxNG(file=f)
+        finally:
+            f.close()
+
+    def test_relaxng_compact_str(self):
+        tree_valid = self.parse('<a><b>B</b></a>')
+        rnc_str = 'element a { element b { "B" } }'
+        schema = etree.RelaxNG.from_rnc_string(rnc_str)
+        self.assertTrue(schema.validate(tree_valid))
 
 
 def test_suite():
@@ -163,6 +204,8 @@ def test_suite():
     suite.addTests([unittest.makeSuite(ETreeRelaxNGTestCase)])
     suite.addTests(
         [make_doctest('../../../doc/validation.txt')])
+    if rnc2rng is not None:
+        suite.addTests([unittest.makeSuite(RelaxNGCompactTestCase)])
     return suite
 
 if __name__ == '__main__':
